% P36SNRimages.m Images showing SNR
%
%
% COPYRIGHT:  Aprend Technology and Robert E. ALvarez, 2013. All rights reserved.
 
%% init vars
post_title = 'P36SNRimages';
dirname = ['G:\Projects\Blog\Posts\' post_title '\'];
addpath([dirname '\code\']);
addpath('G:\Projects\DEsolvePaper\Code\');

%% init the simulated object
thicknesses = [1 -.1 .1]; % gm/cm^2 -- will assume density is equal to 1
nx = 256; ny = 256;
zsiz = nx + 1i*ny; % dimensions of images
z1 = 1 + 1i;
   % boxes coords in unit based coords
zcorners = nx*[ 
    .1*z1, .9*z1; % background material region
    .2*z1,.2*z1 + .2+.6i;  
    .2*z1,.2*z1 + .2+.6i;  % hole for feature
    ];
   % make object out of 3 planes: background, 
   % -feature from background slab, 
   % fill in feature material in background slab  
tplanes = zeros(ny,nx,2); % one plane per background and feature
   % combine background plane with hole for feature
for kt = 1:2
    timg = sim2d(zsiz,'rect',[zcorners(kt,:),thicknesses(kt)]);
    tplanes(:,:,1) =  tplanes(:,:,1) + timg;
end
% add in feature
tplanes(:,:,2) = sim2d(zsiz,'rect',[zcorners(3,:),thicknesses(3)]);
   % reformat into columns (ny*nx,2)
ts_object = reshape(tplanes,ny*nx,2);

%% prep to compute image SNR with an anonymous function
    % get indexes to feature and background regions for statistics
     % call imStatsInPolygon only to get the indexes to points in image
     % zcorners(2,:) (see above) is corners of region with the feature
stats_feat = imStatsInPolygon(zeros(ny,nx),zcorners(2,:)); 
idxs4feat = stats_feat.idxs;
    % corners of region in background
zcorners_for_background = nx*[.2*z1+0.4, 0.2*z1+0.4+ .2+.6i]; 
stats_back = imStatsInPolygon(zeros(ny,nx),zcorners_for_background); 
idxs4back = stats_back.idxs;
   % define anonymous function to compute the SNR^2 
   % for feature with respect to background
SNRfeature = @(img) (mean(img(idxs4feat)) - mean(img(idxs4back)))^2/var(img(idxs4back));

%% compute the basis functions: 
  %  the background material is soft tissue and the feature is cortical bone
dat = BodyMaterialCompositionFunc;
material_back = 'soft_tissue';
ztback = dat.(material_back)(2:end,:);% tables with atomic number and fraction by weight
material_feat = 'bone_cortical'; % 'adipose'; %
ztfeat = dat.(material_feat)(2:end,:);% for background and feature materials 
zAback = 3 + 0i; % background thickness: 3 gm/cm^2 soft tissue and no bone

%% make the incident spectrum and  transmitted spectrum
kV = 120;
[dum,dum,specdat] = XrayTubeSpectrumTasmip(kV,'number_spectrum','clipzeros');
specdat.mus = [xraymu(ztback,specdat.egys) xraymu(ztfeat,specdat.egys)];

%% make the random transmitted data
transmission = exp(-ts_object*specdat.mus');
   % the expected values of the transmitted spectrum
lambdas = bsxfun(@times,transmission,specdat.specnum_norm'); % siz = (ny*nx,nenergy)
N = 10^3; % total number of photons in trasnmitted spectrum
offset2counts = 1/numel(specdat.egys); % add small number so no zeros
   % generate the random data
ns = poissrnd(N*lambdas) + offset2counts; 

%% N image
Ns = sum(ns,2);
imdatN = reshape(Ns,ny,nx);
snrN = SNRfeature(imdatN);

%% Q image
Qs = sum(bsxfun(@times,ns,specdat.egys'),2);
imdatQ = reshape(Qs,ny,nx);
snrQ = SNRfeature(imdatQ);

%% Make the TW optimal image by weighting the full spectrum data at each point in image
   % for a thin object, the optimal weighting function is proportional to
   % diff of the mus at each  energy
dmus = diff(specdat.mus,1,2);
dmus_norm = dmus/sum(dmus);
imTW = reshape(sum(bsxfun(@times,ns,dmus_norm'),2),ny,nx);
snrTW = SNRfeature(imTW);

%% N2Q data -- will process using my A-table method
  % first compute object data
Ethreshold = 35; % keV
[dum,idxthreshold] = min(abs(specdat.egys - Ethreshold));
N2Qs = [sum(ns(:,1:idxthreshold),2) sum(ns(:,(idxthreshold+1):end),2) Qs];
   % 'air data' i.e. no object in beam. Assume has no noise
N2Qs0 = N*[sum(specdat.specnum_norm(1:idxthreshold)) ... 
                   sum(specdat.specnum_norm((idxthreshold+1):end)) ...
                   sum(specdat.specnum_norm.*specdat.egys)];
  % add offset to air data so it matches the offset added to noisy data
N2Qs0(1:2) = N2Qs0(1:2) + offset2counts;
logN2Qs = bsxfun(@minus,log(N2Qs0),log(N2Qs)); % these are positive numbers

%% calibration data -- assume noise free
ncalib_points = 20;
t1s_calib = linspace(-.5,4*thicknesses(1),ncalib_points);        % allow negatives so fewer out of range points            
t2s_calib = linspace(-.3,4*thicknesses(3),ncalib_points); 
idxs = combinator(ncalib_points,2,'p','r'); % permutations taken 2 at a time with repitition
ts_calib = [t1s_calib(idxs(:,1))' t2s_calib(idxs(:,2))'];                   
transmission_calib = exp(-ts_calib*specdat.mus');
lambdas_calib = bsxfun(@times,transmission_calib,specdat.specnum_norm'); % siz = (ncalib steps,nenergy)
ns_calib = N*lambdas_calib + offset2counts; % no noise for calibration data
Qs_calib = sum(bsxfun(@times,ns_calib,specdat.egys'),2);
N2Qs_calib = [sum(ns_calib(:,1:idxthreshold),2) sum(ns_calib(:,(idxthreshold+1):end),2) Qs_calib];
   % air data does not change so use results from previous cell
logN2Qs_calib = bsxfun(@minus,log(N2Qs0),log(N2Qs_calib)); % these are positive numbers
cdat.As = ts_calib;
cdat.logI = logN2Qs_calib;

%% compute covariance of log(N2Q) data from background region of image
covstats.RL = cov(logN2Qs(idxs4back,:));

%% solve for A vector image data
tdat.As = ts_object;
tdat.logI = logN2Qs;
% N2Qdat = PolyASolveEquations(cdat,tdat);
N2Qdat = AtableSolveEquations(cdat,tdat,'stats',covstats);
zAs_back = zcomp(N2Qdat.Astest(idxs4back,:));

%% compute whiten xform -- see my blog posts ebook Eq. 25.23
[V,D] = eig(cov(N2Qdat.Astest(idxs4back,:)));
Pwhiten = V*D^(-.5);
Aswhiten = N2Qdat.Astest*Pwhiten;
zAswhiten = zcomp(Aswhiten);

%% find projection angle for maximum SNR
nprojangles = 100;
projangles = linspace(0,2*pi,nprojangles+1); % will remove 1 so no wraparound
projangles = projangles(1:nprojangles);
snrs = zeros(nprojangles,1);

for kangle = 1:nprojangles
   improj = zdot(zAswhiten,exp(1i*projangles(kangle)));
   snrs(kangle) = SNRfeature(improj);
end
idxpeak = findpeaks(snrs);
assert(~isempty(idxpeak));
imN2Q = reshape(zdot(zAswhiten,exp(1i*projangles(idxpeak(1)))),ny,nx);
snrN2Q = SNRfeature(imN2Q);

%% plot SNRs vs. projection angle
fh = figure;
plot(projangles*180/pi,snrs,'-k','linewidth',1.5);
set(gca,'linewidth',1.5,'fontsize',12);
xlabel('projection angle (degrees)','fontsize',14);
ylabel('SNR^2','fontsize',14);
psn = [125 250 675 500];
set(fh,'color','w','Position',psn)

%% export the figure as png and eps
figno = sprintf('-f%d',fh);
fname = 'SNRvsProjAngle';
   % export as eps
%exportfig(fh,[dirname fname '.eps'],'width',6,'LockAxes',0,'color','cmyk');
    % export as png
set(fh,'papertype','usletter')
print(figno,'-dpng','-cmyk','-r100',[dirname fname '.png']) % resolution 100 dpi

%% plot data before and after whiten xform
zAs_back = zcomp(N2Qdat.Astest(idxs4back,:));
fh = figure;
nframes = 2;
ht = zeros(nframes,1);
fwid = 1/nframes; % fraction of width of figure for subplot
k = 0;
subplot('position',[fwid*k,0,fwid,1])
plot(zAs_back,'.k','linewidth',1.5);
v = axis;
dvs = diff(reshape(v,2,2)); % diff along rows
ht(k+1) = text(v(1) + 0.3*dvs(1),v(3) + 0.95*dvs(2),'As','color','k','fontsize',14);
set(gca,'xtick',[],'ytick',[],'XtickLabel',{},'YtickLabel',{});
k = k+1;
subplot('position',[fwid*k,0,fwid,1]),imshow(imN2Q,[-1 11])
plot(zAswhiten(idxs4back),'.k','linewidth',1.5);
v = axis;
dvs = diff(reshape(v,2,2)); % diff along rows
ht(k+1) = text(v(1) + 0.3*dvs(1),v(3) + 0.95*dvs(2),'As whitened','color','k','fontsize',14);
set(gca,'xtick',[],'ytick',[],'XtickLabel',{},'YtickLabel',{});
psn = [125 250 675 350];
set(fh,'color','w','Position',psn)

%% export the figure as png and eps
figno = sprintf('-f%d',fh);
fname = 'Scatterplots';
   % export as eps
%exportfig(fh,[dirname fname '.eps'],'width',6,'LockAxes',0,'color','cmyk');
    % export as png
set(fh,'papertype','usletter')
print(figno,'-dpng','-cmyk','-r100',[dirname fname '.png']) % resolution 100 dpi

%% display Q, N, N2Q, and TW images
imcells = cell(4,1);
imcells{1} = -imdatQ;
imcells{2} = -imdatN;
imcells{3} = imN2Q;
imcells{4} = -imTW;
fh = DispImagesTiledFromCells(imcells,'nper_row',4,'labels',{'Q', 'N', 'N2Q',  'TW'},'fontsize',14);
psn = [125 125 900 225];
set(fh,'color','w','Position',psn)

%% export the figure as png and eps
figno = sprintf('-f%d',fh);
fname = 'SNRimages';
   % export as eps
exportfig(fh,[dirname fname '.eps'],'width',6,'LockAxes',0,'color','cmyk');
    % export as png
set(fh,'papertype','usletter')
print(figno,'-dpng','-cmyk','-r100',[dirname fname '.png']) % resolution 100 dpi

%% display images from Q, N, N2Q, and Tapiovaara-Wagner optimal
fh = figure;
nframes = 3;
ht = zeros(nframes,1);
fwid = 1/nframes; % fraction of width of figure for subplot
k = 0;
subplot('position',[fwid*k,0,fwid,1])
imsgray(-imdatQ,0)
ht(k+1) = text(128,200,'Q','color','y','fontsize',14);
k = k+1;
subplot('position',[fwid*k,0,fwid,1])
imshow(imN2Q,[-1 11])
ht(k+1) = text(128,200,'N2Q','color','y','fontsize',14);
k = k+1;
subplot('position',[fwid*k,0,fwid,1])
imshow(-imTW,[-4 -1.7])
ht(k+1) = text(128,200,'TW opt','color','y','fontsize',14);
psn = [125 250 675 250];
set(fh,'color','w','Position',psn)

%% export the figure as png and eps
figno = sprintf('-f%d',fh);
fname = 'SNRimages';
   % export as eps
%exportfig(fh,[dirname fname '.eps'],'width',6,'LockAxes',0,'color','cmyk');
    % export as png
set(fh,'papertype','usletter')
print(figno,'-dpng','-cmyk','-r100',[dirname fname '.png']) % resolution 100 dpi

%% display feature and background canceled images
idxcancel = findpeaks(-snrs); % get zeros in SNR
assert(~isempty(idxcancel));
imFeatCancel = reshape(zdot(zAswhiten,exp(1i*projangles(idxcancel(2)))),ny,nx);
ang_air_cancel = angle(mean(zAswhiten(idxs4back))) - pi/2; % assume air has value 0+0i
imBackCancel = reshape(zdot(zAswhiten,exp(1i*ang_back_cancel)),ny,nx);
fh = DispImagesTiledFromCells({imFeatCancel,imBackCancel},'nper_row',2,'labels', ...
    {'cancel feature', 'cancel background'},'fontsize',14);
psn = [125 250 600 300];
set(fh,'color','w','Position',psn)

%% export the figure as png and eps
figno = sprintf('-f%d',fh);
fname = 'FeatCancelledImage';
   % export as eps
exportfig(fh,[dirname fname '.eps'],'width',6,'LockAxes',0,'color','cmyk');
    % export as png
set(fh,'papertype','usletter')
print(figno,'-dpng','-cmyk','-r100',[dirname fname '.png']) % resolution 100 dpi

%% plot incident, soft tissue mu, and transmitted spectrum for figure showing experiment
spectran = specdat;
thickness_soft = 1; % gm/cm^2 of soft tissue
spectran.specnum = spectran.specnum(:).*exp(-thickness_soft*spectran.mus(:,1));

fh = figure;
linewid = 1.5;
subplot(3,1,1)
hspec = plot(specdat.egys,specdat.specnum,'-k','linewidth',linewid);
set(gca,'XtickLabel',{},'YtickLabel',{},'linewidth',linewid)
ylabel('entrance spectrum ','fontsize',14)
% axis([0 70 0 .05])

subplot(3,1,2)
hmu = plot(spectran.egys,spectran.mus(:,1),'-k','linewidth',linewid);
set(gca,'XtickLabel',{},'YtickLabel',{},'linewidth',linewid)
ylabel('attenuation coefficient ','fontsize',14)

subplot(3,1,3)
hspec = plot(spectran.egys,spectran.specnum,'-k','linewidth',linewid);
   % draw lines at bin edges
nbins = 1;
egys_bins =[min(spectran.egys),36,max(spectran.egys)]'; 
hl = line([egys_bins egys_bins]',[zeros(size(egys_bins)) interp1(spectran.egys,spectran.specnum,egys_bins)]'); 
set(hl,'color','k','linewidth',1.)
% axis([0 70 0 .05])
xlabel('energy (keV) ','fontsize',14)
ylabel('exit spectrum ','fontsize',14)
set(gca,'YtickLabel',{},'linewidth',linewid)
psn = [250 100 450 950];
set(fh,'color','w','position',psn)

%% export the image as  eps -- will read into Canvas to make figure
figno = sprintf('-f%d',fh);
fname = 'EntranceExitSpectra';
   % export as eps
exportfig(fh,[dirname fname '.eps'],'width',5,'LockAxes',0,'color','cmyk');

