function [qdatmat,bins_cell] = QuantizeOpt(datmat)
% function [qdatmat,nbins] = QuantizeOpt(datmat)
%  computes the optimally quantized bin indexes for input data dat
%  inputs:
%    datmat: a 2D array of data, will quantize data in each column
%  outputs:
%    qdat: quantized data i.e. bin numbers of optimally binned data
%    bins_cell: a cell array of the bins used for quantizing, 1 cell per column
%  REA 12/6/05-6/27/08

ncols = size(datmat,2);
qdatmat = NaN*ones(size(datmat)); % init with NaNs to catch incomplete data overwrite errors
bins_cell = cell(1,ncols);
for kcol = 1:ncols
  dat = datmat(:,kcol);
  
  uniqs = unique(dat);
  nuniq = numel(uniqs);
  
  if nuniq == 1
    qdatmat(:,kcol) = ones(size(datmat,1),1);
    continue;
  elseif nuniq == 2
    qdatmat(:,kcol) = dat>mean(uniqs); %threshold it to get two values
    continue;
  end
      

  datsort = sort(dat(:));
  
  ndat = numel(datsort);
  
  iqrange = datsort(round(0.75*ndat)) - datsort(round(0.25*ndat));
  if iqrange==0
    iqrange = std(dat);
  end
  
  h = 2*iqrange*ndat^(-1/3); % bin width by freedman-diaconis rule
  

  nbin = ceil(range(datsort)/h);
  bins = (min(datsort)-h) :h: (max(datsort) + h);
  qdat = round(interp1(bins,1:numel(bins),dat));

  qdatmat(:,kcol) = qdat(:);
  bins_cell{kcol} = bins;
end
